@@ -24,6 +24,10 @@ def show_info(args):
     if args.gui:
         from ..gui import launch_gui
 
+        launch_gui()
+    elif args.gui_acr:
+        from ..gui_acr import launch_gui
+
         launch_gui()
     else:
         info = {
