@@ -1,4 +1,5 @@
-from PyQt5.QtWidgets import QHBoxLayout, QWidget, QVBoxLayout, QCheckBox, QLabel, QComboBox
+from PyQt5.QtWidgets import QHBoxLayout, QWidget, QVBoxLayout, QCheckBox, QLabel
+from PyQt5.QtWidgets import QComboBox, QStyledItemDelegate
 
 
 def objective_item(name, rule, callback=None):
@@ -14,6 +15,7 @@ def objective_item(name, rule, callback=None):
     hbox.setContentsMargins(0, 0, 0, 0)
     label = QLabel('rule')
     widget.cb_rule = cb_rule = QComboBox()
+    cb_rule.setItemDelegate(QStyledItemDelegate())
     cb_rule.addItems(['MINIMIZE', 'MAXIMIZE'])
     cb_rule.setCurrentIndex(rule)
     hbox.addWidget(label)
