@@ -1,5 +1,5 @@
 import numpy as np
-from PyQt5.QtWidgets import QVBoxLayout, QHBoxLayout, QWidget, QPushButton
+from PyQt5.QtWidgets import QVBoxLayout, QHBoxLayout, QWidget, QPushButton, QCheckBox
 from PyQt5.QtWidgets import QMessageBox, QComboBox, QLabel, QStyledItemDelegate
 from PyQt5.QtCore import pyqtSignal, QThreadPool
 from PyQt5.QtGui import QFont
@@ -41,7 +41,9 @@ class BadgerOptMonitor(QWidget):
         # self.setAttribute(Qt.WA_DeleteOnClose, True)
 
         # For plot type switching
-        self.x_plot_type = 0  # 0: raw, 1: normalized
+        self.x_plot_y_axis = 0  # 0: raw, 1: normalized
+        self.plot_x_axis = 0  # 0: iteration, 1: time
+        self.x_plot_relative = True
         # Routine info
         self.routine = None
         self.var_names = []
@@ -56,6 +58,7 @@ class BadgerOptMonitor(QWidget):
         self.vars = []
         self.objs = []
         self.cons = []
+        self.ts = []
         # Run optimization
         self.thread_pool = None
         self.routine_runner = None
@@ -76,12 +79,24 @@ class BadgerOptMonitor(QWidget):
         config_bar = QWidget()
         hbox_config = QHBoxLayout(config_bar)
         hbox_config.setContentsMargins(0, 0, 0, 0)
-        label = QLabel('Evaluation History (X) Plot Type')
-        self.cb_plot = cb_plot = QComboBox()
-        cb_plot.setItemDelegate(QStyledItemDelegate())
-        cb_plot.addItems(['Raw', 'Normalized'])
+        label = QLabel('Evaluation History Plot Type')
+        label_x = QLabel('X Axis')
+        self.cb_plot_x = cb_plot_x = QComboBox()
+        cb_plot_x.setItemDelegate(QStyledItemDelegate())
+        cb_plot_x.addItems(['Iteration', 'Time'])
+        label_y = QLabel('Y Axis (Var)')
+        self.cb_plot_y = cb_plot_y = QComboBox()
+        cb_plot_y.setItemDelegate(QStyledItemDelegate())
+        cb_plot_y.addItems(['Raw', 'Normalized'])
+        self.check_relative = check_relative = QCheckBox('Relative')
+        check_relative.setChecked(True)
         hbox_config.addWidget(label)
-        hbox_config.addWidget(cb_plot, 1)
+        # hbox_config.addSpacing(1)
+        hbox_config.addWidget(label_x)
+        hbox_config.addWidget(cb_plot_x, 1)
+        hbox_config.addWidget(label_y)
+        hbox_config.addWidget(cb_plot_y, 1)
+        hbox_config.addWidget(check_relative)
 
         # Set up the monitor
         # Don't set show=True or there will be a blank window flashing once
@@ -211,7 +226,9 @@ class BadgerOptMonitor(QWidget):
         self.btn_stop.clicked.connect(self.run_stop_routine)
 
         # Visualization
-        self.cb_plot.currentIndexChanged.connect(self.select_x_plot_type)
+        self.cb_plot_x.currentIndexChanged.connect(self.select_x_axis)
+        self.cb_plot_y.currentIndexChanged.connect(self.select_x_plot_y_axis)
+        self.check_relative.stateChanged.connect(self.toggle_x_plot_y_axis_relative)
 
     def init_plots(self, routine, data=None, run_filename=None):
         # Parse routine
@@ -316,6 +333,7 @@ class BadgerOptMonitor(QWidget):
         self.vars = []
         self.objs = []
         self.cons = []
+        self.ts = []
         if self.routine_runner and self.routine_runner.run_filename == run_filename:
             self.btn_reset.setDisabled(False)
             self.btn_set.setDisabled(False)
@@ -338,8 +356,9 @@ class BadgerOptMonitor(QWidget):
         for con in self.con_names:
             self.cons.append(data[con])
         self.cons = np.array(self.cons).T.tolist()
+        self.ts = data['timestamp_raw']
 
-        self.update_curves()
+        self.update_curves(auto_range=True)
 
         self.calc_optimals()
         self.btn_del.setDisabled(False)
@@ -417,30 +436,53 @@ class BadgerOptMonitor(QWidget):
         if self.con_names:
             self.plot_con.enableAutoRange()
 
-    def update_curves(self):
-        for i in range(len(self.obj_names)):
-            self.curves_obj[i].setData(np.array(self.objs)[:, i])
+    def update_curves(self, auto_range=False):
+        type_x = self.plot_x_axis
+        type_y = self.x_plot_y_axis
+        relative = self.x_plot_relative
 
-        if self.con_names:
-            for i in range(len(self.con_names)):
-                self.curves_con[i].setData(np.array(self.cons)[:, i])
-
-        if self.x_plot_type:
+        if type_y:
             _vars = norm(np.array(self.vars),
                          self.vranges[:, 0], self.vranges[:, 1])
         else:
             _vars = np.array(self.vars)
+
+        if relative:
+            _vars = _vars - _vars[0]
+
+        if type_x:
+            ts = np.array(self.ts)
+            ts -= ts[0]
+
         for i in range(len(self.var_names)):
-            self.curves_var[i].setData(_vars[:, i])
+            if type_x:
+                self.curves_var[i].setData(ts, _vars[:, i])
+            else:
+                self.curves_var[i].setData(_vars[:, i])
 
-        self.enable_auto_range()
+        for i in range(len(self.obj_names)):
+            if type_x:
+                self.curves_obj[i].setData(ts, np.array(self.objs)[:, i])
+            else:
+                self.curves_obj[i].setData(np.array(self.objs)[:, i])
 
-    def update(self, vars, objs, cons):
+        if self.con_names:
+            for i in range(len(self.con_names)):
+                if type_x:
+                    self.curves_con[i].setData(ts, np.array(self.cons)[:, i])
+                else:
+                    self.curves_con[i].setData(np.array(self.cons)[:, i])
+
+        if auto_range:
+            self.enable_auto_range()
+
+    def update(self, vars, objs, cons, ts):
         self.vars.append(vars)
         self.objs.append(objs)
         self.cons.append(cons)
+        self.ts.append(ts)
 
-        self.update_curves()
+        self.update_curves(auto_range=True)
         self.sig_progress.emit(vars, objs, cons)
 
         # Check critical condition
@@ -538,13 +580,27 @@ class BadgerOptMonitor(QWidget):
             self.ins_con.setValue(ins_var.value())
 
     def ins_drag_done(self, ins):
-        value = np.round(ins.value())
+        self.sync_ins(ins.value())
+
+    def sync_ins(self, pos):
+        if self.plot_x_axis:  # x-axis is time
+            value, idx = self.closest_ts(pos)
+        else:
+            value = idx = np.clip(np.round(pos), 0, len(self.ts) - 1)
         self.ins_obj.setValue(value)
         if self.con_names:
             self.ins_con.setValue(value)
         self.ins_var.setValue(value)
 
-        self.sig_inspect.emit(value)
+        self.sig_inspect.emit(idx)
+
+    def closest_ts(self, t):
+        # Get the closest timestamp in self.ts regarding t
+        ts = np.array(self.ts)
+        ts -= ts[0]
+        idx = np.argmin(np.abs(ts - t))
+
+        return ts[idx], idx
 
     def reset_env(self):
         reply = QMessageBox.question(self,
@@ -569,18 +625,19 @@ class BadgerOptMonitor(QWidget):
             return
 
         idx = pf.pareto_set[0][0]
-        self.ins_obj.setValue(idx)
-        if self.con_names:
-            self.ins_con.setValue(idx)
-        self.ins_var.setValue(idx)
-
+        self.jump_to_solution(int(idx))
         self.sig_inspect.emit(idx)
 
     def jump_to_solution(self, idx):
-        self.ins_obj.setValue(idx)
+        if self.plot_x_axis:  # x-axis is time
+            value = self.ts[idx] -  self.ts[0]
+        else:
+            value = idx
+
+        self.ins_obj.setValue(value)
         if self.con_names:
-            self.ins_con.setValue(idx)
-        self.ins_var.setValue(idx)
+            self.ins_con.setValue(value)
+        self.ins_var.setValue(value)
 
     def set_vars(self):
         df = self.routine_runner.data
@@ -600,15 +657,40 @@ class BadgerOptMonitor(QWidget):
         # QMessageBox.information(
         #     self, 'Set Environment', f'Env vars have been set to {solution}')
 
-    def select_x_plot_type(self, i):
-        self.x_plot_type = i
+    def select_x_axis(self, i):
+        self.plot_x_axis = i
+
+        # Switch the x-axis labels
         if i:
-            _vars = norm(np.array(self.vars),
-                         self.vranges[:, 0], self.vranges[:, 1])
+            self.plot_var.setLabel('bottom', 'time (s)')
+            self.plot_obj.setLabel('bottom', 'time (s)')
+            if self.con_names:
+                self.plot_con.setLabel('bottom', 'time (s)')
         else:
-            _vars = np.array(self.vars)
-        for i in range(len(self.var_names)):
-            self.curves_var[i].setData(_vars[:, i])
+            self.plot_var.setLabel('bottom', 'iterations')
+            self.plot_obj.setLabel('bottom', 'iterations')
+            if self.con_names:
+                self.plot_con.setLabel('bottom', 'iterations')
+
+        # Update inspector line position
+        if i:
+            value = self.ts[int(self.ins_obj.value())] - self.ts[0]
+        else:
+            _, value = self.closest_ts(self.ins_obj.value())
+        self.ins_obj.setValue(value)
+        if self.con_names:
+            self.ins_con.setValue(value)
+        self.ins_var.setValue(value)
+
+        self.update_curves(auto_range=True)
+
+    def select_x_plot_y_axis(self, i):
+        self.x_plot_y_axis = i
+        self.update_curves()
+
+    def toggle_x_plot_y_axis_relative(self):
+        self.x_plot_relative = self.check_relative.isChecked()
+        self.update_curves()
 
     def on_mouse_click(self, event):
         # https://stackoverflow.com/a/64081483
@@ -623,14 +705,7 @@ class BadgerOptMonitor(QWidget):
             flag = flag or self.plot_con.viewRect().contains(coor_con)
 
         if flag:
-            idx = int(np.round(coor_obj.x()))
-
-            self.ins_obj.setValue(idx)
-            if self.con_names:
-                self.ins_con.setValue(idx)
-            self.ins_var.setValue(idx)
-
-            self.sig_inspect.emit(idx)
+            self.sync_ins(coor_obj.x())
 
     def delete_run(self):
         self.sig_del.emit()
